import { describe, expect, it, jest, test, beforeAll, afterAll } from '@jest/globals';
import supertest, { Response } from 'supertest';

const baseUrl = "http://localhost:8000";

describe("Order Manager", () => {

    let cartId: number;
    let itemId: number;

    beforeAll(async () => {
        const cartPayload = {
            name: "Test Cart",
            preis: 10,
            beschreibung: "A test shopping cart",
        };


        const cartRes: Response = await supertest(baseUrl)
            .post("/api/shoppingCart/createShoppingCart")
            .send(cartPayload)
            .set("Content-Type", "application/json")
            .set("Accept", "application/json");

        cartId = cartRes.body.id;

        const itemPayload = {
            name: "Test Item",
            preis: 5,
            beschreibung: "A test item for the shopping cart",
            categoryId: 1 
        };

        const itemRes: Response = await supertest(baseUrl)
            .post("/api/article/addArticle")
            .send(itemPayload)
            .set("Content-Type", "application/json")
            .set("Accept", "application/json");

        itemId = itemRes.body.id;
    });

    describe("Add New Item to Order", () => {
        it("should add a new item to the order and return status 201", async () => {
            const payload = {
                cartId: cartId,
                itemId: itemId,
                quantity: 3
            };

            const res: Response = await supertest(baseUrl)
                .post("/api/order/addNewItemToOrder")
                .send(payload)
                .set("Content-Type", "application/json")
                .set("Accept", "application/json");

            const returnedStatus = res.status;
            const expectedStatus = 200;

            expect(returnedStatus).toEqual(expectedStatus);
        });
    });

    describe("Get Items in Cart", () => {
        it("should retrieve all items in the cart and return status 200", async () => {
            const res: Response = await supertest(baseUrl)
                .get(`/api/order/getItemsInCart?cartId=${cartId}`)
                .set("Content-Type", "application/json");

            const returnedStatus = res.status;
            const expectedStatus = 200;

            expect(returnedStatus).toEqual(expectedStatus);
            expect(Array.isArray(res.body.items)).toBeTruthy();
            expect(res.body.items.length).toEqual(1);
        });
    });

    describe("Update Product Quantity", () => {
        it("should update the quantity of an item in the cart and return status 204", async () => {
            const payload = {
                cartId: cartId,
                articleId: itemId,
                newQuantity: 5
            };

            const res: Response = await supertest(baseUrl)
                .put("/api/order/updateProductQuantity")
                .send(payload)
                .set("Content-Type", "application/json");

            const returnedStatus = res.status;
            const expectedStatus = 200;

            expect(returnedStatus).toEqual(expectedStatus);
        });
    });

    describe("Delete Item from Shopping Cart", () => {
        it("should delete an item from the shopping cart and return status 204", async () => {

            const payload = {
                itemId: itemId,
                cartId: cartId
            }

            const res: Response = await supertest(baseUrl)
                .delete(`/api/order/deleteItemFromShoppingCart`)
                .send(payload)
                .set("Content-Type", "application/json");

            const returnedStatus = res.status;
            const expectedStatus = 200;

            expect(returnedStatus).toEqual(expectedStatus);
        });
    });

    afterAll(async () => {
        let resItem = await supertest(baseUrl).delete(`/api/article/deleteArticle?id=${itemId}`);
        const resCart: Response = await supertest(baseUrl)
            .delete(`/api/shoppingCart/deleteShoppingCart?id=${cartId}`)
            .set("Content-Type", "application/json");
    })
});
